/*
 * tmp-audit v.0.2
 * airport man <airportman@poc.csoft.net>
 *
 * Feel free to redistribute the source code in any form,
 * modified or not.
 *
 * Original source [http://poc.csoft.net/code/tmp-audit/]
 */
#include "tmp-audit.h"

int opt_dheader=1, opt_clear=0, opt_log=0;
char log_file[255], tmp_dir[255];
long int tvsec=DEFAULT_SEC, tvusec=DEFAULT_USEC;


/*
  dosettimer() - set interval timer
  This sets a real timer (sends an SIGALRM back) using values from
  tvsec and tvusec.  I prefer to use the hardware timer here - change
  it if you want (don't forget to change it to SIGVTALRM if you want
  a virtual timer instead)
*/
void
dosettimer()
{
  struct timeval tv;
  struct itimerval itval;

  tv.tv_sec=tvsec;
  tv.tv_usec=tvusec;
  itval.it_interval=tv;
  itval.it_value=tv;

  if( setitimer(ITIMER_REAL,&itval,NULL) <0 )
    perror("setitimer()");
}

/*
  dosetsignals() - set up signal handlers
  Should need to run this only once, it nulls sa_flags to
  remove SA_ONESHOT (or SA_RESETHAND), set by signal().
*/
void
dosetsignals()
{
  struct sigaction sa;
  signal(SIGALRM, &doscan);
  sigaction(SIGALRM, NULL, &sa);
  sa.sa_flags = 0;
  sigaction(SIGALRM, &sa, NULL);
}

/*
  dorep() - print individual file data
  Undefine OPT_STAT if you don't want to stat() the file and
  check for UIDs - that could speed up the process a bit if
  you just need file names.
*/
void
dorep(name)
     char* name;
{
  char f_file[255];
  struct stat dstat;

  snprintf(f_file,sizeof(f_file),"%s/%s",tmp_dir,name);

  printf("%25s ", name);
  if( !access(f_file,F_OK) ) {
#ifdef OPT_STAT
    if( stat(f_file,&dstat) <0 )
      perror("stat()");
    else {
      static struct passwd* passwd;
      if(!(passwd=getpwuid(dstat.st_uid)))
	printf("%10o [%d/%d]",
	       dstat.st_mode, dstat.st_uid, dstat.st_gid);
      else
	printf("%10o [%s/%d]",
	       dstat.st_mode, passwd->pw_name, dstat.st_gid);
    }
#ifdef OPT_SIZE
    printf("%10d",(int)dstat.st_size);
#endif /* OPT_SIZE */

#endif /* OPT_STAT */
    putchar('\n');
  }
}

/*
  doscan() - scan directory tmp_dir for changes.
  A `change' is when the number of elements last scan != current scan.
  This is not reliable with slow refresh rates - for example in a case
  where a file deleted and a file created between two scans.  But having
  to compare strings between each scan would be much worse.  See the
  README for a workaround.
 */
void
doscan()
{
  struct dirent **den;
  static int i=0, i2=0, il;
  static short int changenum=0;

  signal(SIGALRM,SIG_IGN);
  i=scandir(tmp_dir,&den,0,0);

#ifndef ALWAYS_REFRESH
  if(i != i2) {
#else
  if(1) {
#endif /* ALWAYS_REFRESH */
    if(opt_clear)
      system(CLEAR_LOC);
    if(opt_dheader)
#ifdef HEADER_BEEP
      printf("\a\n[%s - Change %i]\n",tmp_dir,changenum);
#else
      printf("\n[%s - Change %i]\n",tmp_dir,changenum);
#endif /* HEADER_BEEP */

    for(il=0;il<i;il++)
      if( (strncmp(".",den[il]->d_name,2) != 0) &&
	  (strncmp("..",den[il]->d_name,3)) )
	dorep(den[il]->d_name);
    i2=i;
    changenum++;
  }
  dosettimer();
  dosetsignals();
}

int
main(argc,argv)
     int argc;
     char** argv;
{
  int c, o1=0;

  /* parse command-line parameters */
  while( (c=getopt(argc,argv,"d:s:t:lch")) != EOF ) {
    switch(c) {
    case 'd':
      strncpy(tmp_dir,optarg,sizeof(tmp_dir));
      o1=1; break;
    case 't':
      tvusec = atol(optarg); break;
    case 's':
      tvsec = atol(optarg); break;
    case 'l':
      opt_dheader=0; break;
    case 'c':
      opt_clear=1; break;
    case 'h':
      printf(USAGE,argv[0],DEFAULT_SEC,DEFAULT_USEC);
      exit(0);
    case 'v':
      printf("%s\n",VERSION);
      exit(0);
    }
  }
  if(!o1) strcpy(tmp_dir,DEFAULT_DIR);

  dosetsignals();
  doscan();
  while(1) { pause(); }
}
